/*
 * This program was written by LJ LongWing (lj@remedylegacy.com) and he holds the copyright.
 * This tool is provided for free and can be distributed at will, for private or commercial use.
 * Disclaimer of Warranty.
		THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
		THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR 
		IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. 
		THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, 
		YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

	Limitation of Liability.
		IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO 
		MODIFIES AND/OR CONVEYS THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, 
		INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED 
		TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM 
		TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
 */

package com.apl;

import java.io.*;
import java.net.MalformedURLException;
import java.net.ServerSocket;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.*;

import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

import com.bmc.arsys.api.*;
import com.bmc.arsys.util.ARUtilEgcp;
import com.bmc.thirdparty.org.apache.commons.configuration.ConfigurationException;
import com.bmc.thirdparty.org.apache.commons.configuration.ConversionException;
import com.bmc.thirdparty.org.apache.commons.configuration.PropertiesConfiguration;

public class Daemon {
	static String intro = "A Programming Legacy - http://remedylegacy.com";
	static String app = "APL Load Balancer Monitor";
	static String version = "1.3";

	static String monitor = "";
	static String user = "";
	static String pass = "";
	static String server = "";
	static String configFileName = "config.properties";
	static String RASValue = "";
	static String url = "";
	static String monitorValue = "";
	static String serviceForm = "APL:LBMonitor";
	
	static int tcpPort = 0;

	static ARServerUser conn;

	static int monitorPort = 3636;
	static int downCount = 0;
	static int downThreshold = 3;
	static int checkOverlapCount = 0;

	static long lastCheck = 0;
	static long checkInterval = 0;
	static long checkIntervalWhenUp = 60;
	static long checkIntervalWhenDown = 10;
	static long lastConfigCheck = 0;
	static volatile long onlineStartCheck = 0;

	static Properties config;

	static volatile boolean alive = false;
	static volatile boolean checkStarted = false;
	static boolean useRAS = false;

	static Thread monitorThread = null;

	static ServerSocket serverSocket = null;

	private static Logger logger = Logger.getLogger(Daemon.class);

	public static void main(String[] args) throws Exception {
		int i = 0;
		while (i < args.length && args[i].startsWith("-")) {
			String arg = args[i++];

			if (arg.equals("-c")) {
				if (i < args.length)
					configFileName = args[i++];
			}
		}	
		
		PropertyConfigurator.configure(configFileName);
		logger.info(intro);
		logger.info(app + " " + version);

		processCfg(args);
		monitorThread = new SocketThread(monitorPort);
		monitorThread.start();
		while (true) {
			checkServer(args);
			try {
				Thread.sleep(100);
			} catch (InterruptedException e1) {
				e1.printStackTrace();
			}
		}
	}

	static void checkServer(String[] args) {
		if (alive && downCount == 0 && checkOverlapCount == 0) {
			checkInterval = checkIntervalWhenUp;
		} else {
			checkInterval = checkIntervalWhenDown;
		}

		long currentCheck = System.currentTimeMillis()/1000;
		long nextCheck = lastCheck + checkInterval;

		if (currentCheck >= nextCheck) {
			lastCheck = currentCheck;

			File configFile = new File(configFileName);
			long configFileModify = configFile.lastModified();

			if (configFileModify > lastConfigCheck)
				processCfg(args);

			if (checkStarted) {
				checkOverlapCount++;
			} else {
				checkOverlapCount = 0;
				checkStarted = true;
				
				if (monitor.equalsIgnoreCase("remedy")) {
					conn = new ARServerUser(user, pass, null, server, tcpPort);
					new ServerCheckThread(conn).start();
				} else if (monitor.equalsIgnoreCase("midtier")) {
					new ServerCheckThread(url, user, pass).start();
				}
			}

			if (checkOverlapCount == downThreshold) {
				fail("Server Check has not reported back in " + ((System.currentTimeMillis()/1000)-onlineStartCheck) + " seconds", currentCheck);
				checkOverlapCount = 0;
			}
		}
	}

	static void processCfg(String[] args) {
		int i = 0;

		while (i < args.length && args[i].startsWith("-")) {
			String arg = args[i++];

			if (arg.equals("-c")) {
				if (i < args.length)
					configFileName = args[i++];
			}
		}		

		logger.info("Reading config file: " + configFileName);

		PropertiesConfiguration config = null;
		try {
			config = new PropertiesConfiguration(configFileName);
		} catch (ConfigurationException e) {
			logger.error("Error Reading config file: " + configFileName + " please correct all config file errors to ensure config file is being fully read");
			logger.error(e.getMessage());
			System.exit(1);
		}

		if (config.getString("monitor") != null && !config.getString("monitor").isEmpty())
			monitor = config.getString("monitor");
		if (config.getString("serviceForm") != null && !config.getString("serviceForm").isEmpty())
			serviceForm = config.getString("serviceForm");
		if (config.getString("server") != null && !config.getString("server").isEmpty())
			server = config.getString("server");
		if (config.getString("user") != null && !config.getString("user").isEmpty())
			user = config.getString("user");
		if (config.getString("pass") != null && !config.getString("pass").isEmpty())
			pass = config.getString("pass");
		if (config.getString("useRAS") != null && !config.getString("useRAS").isEmpty())
			useRAS = config.getBoolean("useRAS");
		if (config.getString("RASValue") != null && !config.getString("RASValue").isEmpty())
			RASValue = config.getString("RASValue");
		if (config.getString("url") != null && !config.getString("url").isEmpty())
			url = config.getString("url");

		try {
			tcpPort = config.getInt("tcpPort");
		} catch (ConversionException e) {
			logger.error("Error Reading tcpPort config parameter, using default value for Port Mapper");
			logger.error(e.getMessage());
		}
		try {
			monitorPort = config.getInt("monitorPort");
		} catch (ConversionException e) {
			logger.error("Error Reading monitorPort config parameter, using default value of " + monitorPort);
			logger.error(e.getMessage());
		}
		try {
			checkIntervalWhenUp = config.getInt("checkIntervalWhenUp");
		} catch (ConversionException e) {
			logger.error("Error Reading checkIntervalWhenUp config parameter, using default value of " + checkIntervalWhenUp);
			logger.error(e.getMessage());
		}
		try {
			checkIntervalWhenDown = config.getInt("checkIntervalWhenDown");
		} catch (ConversionException e) {
			logger.error("Error Reading checkIntervalWhenDown config parameter, using default value of " + checkIntervalWhenDown);
			logger.error(e.getMessage());
		}
		try {
			downThreshold = config.getInt("downThreshold");
		} catch (ConversionException e) {
			logger.error("Error Reading downThreshold config parameter, using default value of " + downThreshold);
			logger.error(e.getMessage());
		}

		if (useRAS && RASValue.isEmpty()) {
			RASValue = new GetRASPassword(server, user, pass, tcpPort).run();

			if (RASValue.startsWith("ERROR:")) {
				logger.error(RASValue);
				RASValue = "";
			} else {
				config.setProperty("user", "");
				config.setProperty("pass", "");
				config.setProperty("RASValue", RASValue);

				try {
					config.save();
				} catch (ConfigurationException e) {
					logger.error("Error updating config file with RAS Value: " + e.getMessage() + "\r\n");
				}
			}
		}

		if (monitor.equalsIgnoreCase("remedy")) {
			//output("Monitoring Remedy Server: '" + server + "'");
			monitorValue = server;
		} else if (monitor.equalsIgnoreCase("midtier")) {
			//output("Monitoring Mid-Tier URL: '" + url + "'");
			monitorValue = url;
		} else {
			logger.error("You must either monitor 'remedy' or 'midtier'");
			System.exit(1);
		}

		if (server.isEmpty() && monitor.equalsIgnoreCase("remedy")) {
			logger.error("Cannot run with a blank server name if monitoring remedy, exiting");
			System.exit(1);
		}

		if (useRAS && !RASValue.isEmpty()) {
			logger.info("Using RAS Value to connect");
			user = "Remedy Application Service";
			pass = new ARUtilEgcp().GCEUtilApp(RASValue);
		}

		if (user.isEmpty()) {
			logger.error("ERROR: Cannot run with a blank user name, exiting");
			System.exit(1);
		}

		lastConfigCheck = System.currentTimeMillis();
	}

	static void fail(String s, long startTime) {
		//sendMessage("Query Failed");
		long totalTime = (System.currentTimeMillis()/1000)-startTime;
		logger.error("Server Down (" + monitorValue + "): " + totalTime + " second(s) to check: " + s);

		downCount++;

		if (downCount == downThreshold && alive) {
			alive = false;
			logger.error("Taking monitoring port down because the threshold has been exceeded");
			try {
				serverSocket.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
	}

	static String encodeURL(String urlS) {
		String encodedUrl = null;
		try {
			URL url = new URL(urlS);
			String urlProtocol = url.getProtocol();
			String urlUserInfo = url.getUserInfo();
			String urlHost = url.getHost();
			int urlPort = url.getPort();
			String urlPath = url.getPath();
			String urlQuery = url.getQuery();
			String urlRef = url.getRef();

			if (urlRef != null && !urlRef.isEmpty()) {
				urlQuery += "#" + urlRef;
				urlRef = null;
			}

			URI uri = new URI(urlProtocol, urlUserInfo, urlHost, urlPort, urlPath, urlQuery, urlRef);
			encodedUrl = uri.toASCIIString();
		} catch (MalformedURLException e) {
			encodedUrl = "IOException formatting URL: " + e.getMessage();
		} catch (URISyntaxException e) {
			encodedUrl = "URISyntaxException formatting URL: " + e.getMessage();
		}

		return encodedUrl;
	}
}
