/*
 * This program was written by LJ LongWing (lj@remedylegacy.com) and he holds the copyright.
 * This tool is provided for free and can be distributed at will, for private or commercial use.
 * Disclaimer of Warranty.
		THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW. EXCEPT WHEN OTHERWISE STATED IN WRITING 
		THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR 
		IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. 
		THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE PROGRAM PROVE DEFECTIVE, 
		YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

	Limitation of Liability.
		IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO 
		MODIFIES AND/OR CONVEYS THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY GENERAL, SPECIAL, 
		INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED 
		TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM 
		TO OPERATE WITH ANY OTHER PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES. 
 */

package com.apl;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.StringReader;

import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.Socket;
import java.net.SocketTimeoutException;
import java.net.URL;

import java.security.cert.X509Certificate;

import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLEngine;
import javax.net.ssl.SSLSession;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509ExtendedTrustManager;

import org.apache.log4j.Logger;

import com.bmc.arsys.api.*;

public class ServerCheckThread extends Thread {
	private ARServerUser conn;

	private String urlS = "";
	private String user = "";
	private String pass = "";

	private long startTime = 0;
	private long totalTime = 0;

	private boolean remedyServer = false;
	private boolean webServer = false;

	private static Logger logger = Logger.getLogger(Daemon.class);

	public ServerCheckThread(ARServerUser conn) {
		super("ARServerOnlineCheckThread");
		this.conn = conn;
		remedyServer = true;
	}

	public ServerCheckThread(String url, String user, String pass) {
		super("ARServerOnlineCheckThread");
		this.urlS = url;
		this.user = user;
		this.pass = pass;
		webServer = true;
	}

	public void run() {
		startTime = System.currentTimeMillis()/1000;
		Daemon.onlineStartCheck = startTime;

		if (remedyServer) {
			try {
				conn.login();
			} catch (ARException e) {
				String message = "Error connecting to " + conn.getServer();
				if (conn.getPort() > 0)
					message += ":" + conn.getPort();
				Daemon.fail(message + ": " + e.getMessage(), startTime);
				Daemon.checkStarted = false;
				return;
			}

			Entry inputEntry = new Entry();
			//service field
			inputEntry.put(new Integer(700000001), new Value("ExerciseServer", DataType.CHAR));
			int outputFields[] = new int[] {700000002};
			Entry outputEntry = new Entry();
			try {
				outputEntry = conn.executeService(Daemon.serviceForm, null, inputEntry, outputFields);
			} catch (ARException e) {
				Daemon.fail(e.getMessage(), startTime);
				Daemon.checkStarted = false;
				return;
			}

			if (outputEntry == null || outputEntry.size() == 0 || !outputEntry.containsKey(700000002)) {
				Daemon.fail("Service Call did not return a result", startTime);
				Daemon.checkStarted = false;
				return;
			}

			Value value = outputEntry.get(700000002);
			if (value != null) {
				String result = value.toString();

				if (result != null && !result.equals("")) {
					if (!result.equalsIgnoreCase("OK")) {
						Daemon.fail("Service Call Returned the following error: " + result, startTime);
						Daemon.checkStarted = false;
						return;
					}
				} else {
					Daemon.fail("Service Call returned a blank result", startTime);
					Daemon.checkStarted = false;
					return;
				}
			} else {
				Daemon.fail("Service Call did not return a result", startTime);
				Daemon.checkStarted = false;
				return;
			}

			conn.logout();
		} else if (webServer) {
			boolean working = true;

			if (!urlS.endsWith("/"))
				urlS += "/";
			String urlAppended = urlS + Daemon.serviceForm + "/?username=" + user + "&pwd=" + pass;

			//System.out.println("Raw: " + urlAppended);			

			urlAppended = Daemon.encodeURL(urlAppended);

			//System.out.println("Encoded: " + urlAppended);			


			URL url;
			HttpURLConnection webConn;
			try {
				url = new URL(urlAppended);
				webConn = (HttpURLConnection) url.openConnection();
				//set the timeout values to 30 seconds
				webConn.setConnectTimeout(30 * 1000);
				webConn.setReadTimeout(30 * 1000);
			} catch (MalformedURLException e) {
				String message = "MalformedURLException: " + e.getMessage();
				Daemon.fail(message, startTime);
				Daemon.checkStarted = false;
				return;
			} catch (SocketTimeoutException e) {
				String message = "SocketTimeoutException opening URL: " + e.getMessage();
				Daemon.fail(message, startTime);
				Daemon.checkStarted = false;
				return;
			} catch (IOException e) {
				String message = "IOException opening URL: " + e.getMessage();
				Daemon.fail(message, startTime);
				Daemon.checkStarted = false;
				return;
			}

			if (urlS.startsWith("https")) {
				try {
					TrustManager[] trustAllCerts = new TrustManager[] { new X509ExtendedTrustManager() {
						public java.security.cert.X509Certificate[] getAcceptedIssuers() {
							return null;
						}

						public void checkClientTrusted(X509Certificate[] certs, String authType) {}

						public void checkServerTrusted(X509Certificate[] certs, String authType) {}

						@Override
						public void checkClientTrusted(X509Certificate[] arg0, String arg1, Socket arg2) {}

						@Override
						public void checkClientTrusted(X509Certificate[] arg0, String arg1, SSLEngine arg2) {}

						@Override
						public void checkServerTrusted(X509Certificate[] arg0, String arg1, Socket arg2) {}

						@Override
						public void checkServerTrusted(X509Certificate[] arg0, String arg1, SSLEngine arg2) {}
					} };

					// Install the all-trusting trust manager
					SSLContext sc = SSLContext.getInstance("SSL");
					sc.init(null, trustAllCerts, new java.security.SecureRandom());
					HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());

					// Create all-trusting host name verifier
					HostnameVerifier allHostsValid = new HostnameVerifier() {
						public boolean verify(String hostname, SSLSession session) {
							return true;
						}
					};

					// Install the all-trusting host verifier
					HttpsURLConnection.setDefaultHostnameVerifier(allHostsValid);
				} catch (Exception e) {}
			}

			webConn.setDoOutput(true);
			HttpURLConnection.setFollowRedirects(true);

			try {
				webConn.connect();
			} catch (IOException e) {
				String message = "IOException connecting to URL: " + e.getMessage();
				Daemon.fail(message, startTime);
				Daemon.checkStarted = false;
				return;
			}

			int responseCode;
			String responseMessage;
			String responseBody;
			try {
				responseCode = webConn.getResponseCode();
				responseMessage = webConn.getResponseMessage();
				InputStream is  = webConn.getInputStream();

				BufferedReader br = null;
				StringBuilder sb = new StringBuilder();

				String line;
				try {
					br = new BufferedReader(new InputStreamReader(is));
					while ((line = br.readLine()) != null)
						sb.append(line);
				} catch (IOException e) {
				} finally {
					if (br != null) {
						try {
							br.close();
						} catch (IOException e) {}
					}
				}				
				responseBody = sb.toString();

				//System.out.println(responseBody);

				try {
					// the HTML to convert
					StringReader in = new StringReader(responseBody);
					Html2Text parser = new Html2Text();
					parser.parse(in);
					in.close();
					responseBody = parser.getText();
				} catch (Exception e) {}

				responseBody = responseBody.replaceAll("BMCRemedy.*- Error page", "");

				while (responseBody.contains("  ")) {
					responseBody = responseBody.replaceAll("  ", " ");
				}

				int responseBodyMaxLength = 150;
				if (responseBody.length() > responseBodyMaxLength)
					responseBody = responseBody.substring(0, responseBodyMaxLength) + "...";
			} catch (IOException e) {
				String message = "IOException getting response: " + e.getMessage();
				Daemon.fail(message, startTime);
				Daemon.checkStarted = false;
				return;
			}

			URL newUrl = webConn.getURL();

			String originalHost = url.getHost();
			String originalPath = url.getPath();
			originalPath = originalPath.replaceAll(":", "%3A");

			String newHost = newUrl.getHost();
			String newPath = newUrl.getPath();

			if (responseCode != 200 
					|| url.equals(newUrl)
					|| !originalHost.equalsIgnoreCase(newHost)
					|| !newPath.contains(originalPath)
					|| !newUrl.toString().contains("cacheid")
					) {
				working = false;
				//System.out.println(responseCode);
				//System.out.println(url);
				//System.out.println(originalHost + " : " + newHost);
				//System.out.println(originalPath + " : " + newPath);
			}

			if (!working) {
				String message = "Error getting proper response from Web Server: " + responseCode + ": " + responseMessage + ": " + responseBody;
				Daemon.fail(message, startTime);
				Daemon.checkStarted = false;
				return;
			}
		}

		totalTime = (System.currentTimeMillis()/1000)-startTime;

		if (!Daemon.alive) {
			logger.info("Putting monitoring port up because server reports online");
			Daemon.alive = true;
		}

		logger.info("Server Up (" + Daemon.monitorValue + "): " + totalTime + " second(s) to check");

		Daemon.downCount = 0;
		Daemon.checkStarted = false;
	}
}